require 'spec_helper'
require 'honest_pubsub/cli'

describe HonestPubsub::CLI do
  let(:cli) { HonestPubsub::CLI.instance }
  after(:each) do
    cli.instance_variable_set(:@subscribers, nil)
    cli.instance_variable_set(:@pidfile, nil)
    cli.instance_variable_set(:@require_path, nil)
  end

  describe '#parse' do
    let(:options) { [] }
    before { cli.parse(options) }

    context "pidfile" do
      it "works when nothing is passed" do
        expect(cli.pidfile).to be_nil
      end

      context "passed in as -P" do
        let(:options) { [ "-P", "mypidfile.pid"]}
        it "parses" do
          expect(cli.pidfile).to eq('mypidfile.pid')
        end
      end

      context "passed in as --pidfile" do
        let(:options) { [ "--pidfile", "mypidfile.pid"]}
        it "parses" do
          expect(cli.pidfile).to eq('mypidfile.pid')
        end
      end
    end

    context "only" do
      it "works when nothing is passed" do
        expect(cli.subscribers).to be_nil
      end

      context "passed in as -o" do
        let(:options) { [ "-o", "Foobar,Doobar"]}
        it "parses" do
          expect(cli.subscribers).to eq([ 'Foobar', 'Doobar'])
        end
      end

      context "passed in as --only" do
        let(:options) { [ "--only", "Foobar,Doobar"]}
        it "parses" do
          expect(cli.subscribers).to eq([ 'Foobar', 'Doobar'])
        end
      end
    end

    context "require" do
      it "works when nothing is passed" do
        expect(cli.require_path).to eq(".")
      end

      context "passed in as -r" do
        let(:options) { [ "-r", "/path/to/my/file"]}
        it "parses" do
          expect(cli.require_path).to eq("/path/to/my/file")
        end
      end

      context "passed in as --require" do
        let(:options) { [ "--require", "/path/to/my/file"]}
        it "parses" do
          expect(cli.require_path).to eq("/path/to/my/file")
        end
      end
    end
  end

  describe '#run' do
    describe "high-level steps to be completed" do
      before {
        allow(cli).to receive(:load_environment)
        allow(cli).to receive(:write_pidfile)
        allow(cli).to receive(:load_subscribers)
        cli.run
      }

      it "runs" do
        expect(cli).to have_received(:load_environment)
        expect(cli).to have_received(:write_pidfile)
        expect(cli).to have_received(:load_subscribers)
      end
    end
  end

  describe '#write_pidfile' do
    before {
      cli.pidfile = pidfile
      allow(File).to receive(:open)
      cli.send(:write_pidfile)
    }
    context "no pidfile passed" do
      let(:pidfile) { nil }
      it "doesn't create a pidfile" do
        expect(File).to_not have_received(:open)
      end
    end

    context "pidfile path is supplied" do
      let(:pidfile) { "/path/to/pidfile" }
      it 'creates file' do
        expect(File).to have_received(:open).with(pidfile, 'w')
      end
    end
  end

  describe '#load_subscribers' do
    let(:subscribers) { nil }
    let(:subscriber_server) { double('subscriber_server')}
    before {
      cli.subscribers = subscribers
      allow(subscriber_server).to receive(:start)
      allow(HonestPubsub::Server::SubscriberServer).to receive(:new).and_return(subscriber_server)
      cli.send(:load_subscribers)
    }

    context "No subscribers passed through CLI" do
      let(:all_subscribers) { HonestPubsub::Subscriber.class_variable_get(:@@registered_subscribers) }
      it "loads all subscribers" do
        expect(all_subscribers).to include(MyTestSubscriber1)
        expect(all_subscribers).to include(MyTestSubscriber2)
        expect(all_subscribers).to include(MyTestSubscriber3)
        expect(HonestPubsub::Server::SubscriberServer).to have_received(:new).with(all_subscribers)
        expect(subscriber_server).to have_received(:start)
      end
    end

    context "Subscribers passed through CLI" do
      let(:subscribers) { ['MyTestSubscriber1', 'MyTestSubscriber2'] }
      it "loads all subscribers" do
        expect(HonestPubsub::Server::SubscriberServer).to have_received(:new).with([MyTestSubscriber1, MyTestSubscriber2])
        expect(subscriber_server).to have_received(:start)
      end
    end
  end

end
