require 'spec_helper'

describe HonestPubsub::Subscriber do
  before(:each) { Object.const_set :Klass, Class.new(HonestPubsub::Subscriber) }
  after(:each) { Object.send :remove_const, :Klass }

  describe "#routing_key" do
    subject { HonestPubsub::Subscriber.new(routing_data, properties, {}).routing_key }

    let(:properties) { {} }

    context "Routing key is 'honest.test.route'" do
      let(:routing_data) { {routing_key: "honest.test.route"} }
      
      it { should eq "honest.test.route" }
    end
  end

  describe "self.validates_payload_with" do
    before do
      validators.each do |validator|
        Klass.validates_payload_with validator
      end
    end

    context "One validator" do
      let(:validators){ [lambda{|payload| true}] }
      it {
        expect(Klass.new({},{}, {}).payload_validators.length).to eq 1 }
    end

    context "Two validators" do
      let(:validators){ [lambda{|payload| true}, lambda{|payload| true}] }
      it { expect(Klass.new({},{}, {}).payload_validators.length).to eq 2  }
    end
  end

  describe '#valid_payload?' do
    context
    subject { Klass.new({},{}, {}).valid_payload?({}) }

    before do
      validators.each do |validator|
        Klass.validates_payload_with validator
      end
    end

    context "Two validators returning true" do
      let(:validators){ [lambda{|payload| true}, lambda{|payload| true}] }
      it { should be true }
    end

    context "One validator returning true one returning false" do
      let(:validators){ [lambda{|payload| false}, lambda{|payload| true}] }
      it { should be false }
    end

    context "No validators" do
      let(:validators){ [] }
      it { should be true }
    end

    context "validators as symbol" do
      context "method returns true" do
        before {
          Klass.class_variable_set(:@@foo_bar_called, false)
          Klass.send :define_method,  :foo_bar do |payload|
            Klass.class_variable_set(:@@foo_bar_called, true)
            true
          end
        }
        let(:validators) { [lambda{|payload| true}, :foo_bar] }
        it {
          should be true
          expect(Klass.class_variable_get(:@@foo_bar_called)).to be true
        }

      end

      context "method returns false" do
        before {
          Klass.class_variable_set(:@@foo_bar_called, false)
          Klass.send :define_method,  :foo_bar do |payload|
            Klass.class_variable_set(:@@foo_bar_called, true)
            false
          end
        }
        let(:validators) { [lambda{|payload| true}, :foo_bar] }
        it {
          should be false
          expect(Klass.class_variable_get(:@@foo_bar_called)).to be true
        }

      end

      context "proc returns false" do
        before {
          Klass.class_variable_set(:@@foo_bar_called, false)
          Klass.send :define_method,  :foo_bar do |payload|
            Klass.class_variable_set(:@@foo_bar_called, true)
            true
          end
        }
        let(:validators) { [lambda{|payload| false}, :foo_bar] }
        it "method shouldn't be called" do
          should be false
          expect(Klass.class_variable_get(:@@foo_bar_called)).to be false
        end

      end
    end
  end

  describe "#validate_routing_key_name" do
    it { expect(Klass.send(:validate_routing_key_name, 'abcdef')).to eq(true) }
    it { expect(Klass.send(:validate_routing_key_name, '')).to eq(true) }
    it { expect(Klass.send(:validate_routing_key_name, 'abcdef.abcdef')).to eq(true) }
    it { expect(Klass.send(:validate_routing_key_name, 'abcdef.abcdef.asd')).to eq(true) }
    it { expect(Klass.send(:validate_routing_key_name, 'abcdef.abcdef/')).to eq(false) }
    it { expect(Klass.send(:validate_routing_key_name, 'abcdef.abcdef.a123')).to eq(false) }
    it { expect(Klass.send(:validate_routing_key_name, 'abcdef.')).to eq(false) }
    it { expect(Klass.send(:validate_routing_key_name, 'abcAf')).to eq(false) }
  end

  describe "#generated_queue_name" do
    before(:each) { allow(HonestPubsub::Configuration).to receive(:application_name).and_return("app_name")}

    it { expect(Klass.send(:generated_queue_name, 'abcdef', 'queue_name')).to eq('queue_name') }
    it { expect(Klass.send(:generated_queue_name, 'abcdef', nil)).to eq('app_name_abcdef') }
    it { expect(Klass.send(:generated_queue_name, 'abcdef.abcd', nil)).to eq('app_name_abcdef_abcd') }
    it { expect(Klass.send(:generated_queue_name, 'a.b.c', nil)).to eq('app_name_a_b_c') }
  end

  describe '.subscribe_to' do
    before(:each) { allow(HonestPubsub::Configuration).to receive(:application_name).and_return("app_name")}
    context 'routing_key without queue_name' do
      before { Klass.subscribe_to 'a.b.c' }
      it 'sets the routing key and queue name' do
        expect(Klass.subscribed_key).to eq('a.b.c')
        expect(Klass.subscribed_queue).to eq('app_name_a_b_c')
      end
    end

    context 'routing_key with queue_name' do
      before { Klass.subscribe_to 'a.b.c', on: 'foo_bar' }
      it 'sets the routing key and queue name' do
        expect(Klass.subscribed_key).to eq('a.b.c')
        expect(Klass.subscribed_queue).to eq('foo_bar')
      end
    end

    context 'invalid routing key' do
      before {  }
      it 'sets the routing key and queue name' do
        expect {
          Klass.subscribe_to 'a.b.c.', on: 'foo_bar'
        }.to raise_error(ArgumentError)
      end
    end
  end
end